import os
from typing import List

import typer
from pymisp import PyMISP
from typer import Typer, Option, Argument

from vt2m.lib import lib
from vt2m.lib.output import debug, info, warning, error
from vt2m.subcommands import notifications, retrohunts
from rich.logging import RichHandler
from logging import basicConfig
app = Typer()
state = {
    "verbose": False,
    "quiet": False
}


@app.command()
def query(
        query: str = Argument(..., help="VirusTotal Query"),
        uuid: str = Option(..., "--uuid", "-u", help="MISP event UUID"),
        url: str = Option(None, "--url", "-U", help="MISP URL - can be passed via MISP_URL env"),
        key: str = Option(None, "--key", "-K", help="MISP API Key - can be passed via MISP_KEY env"),
        vt_key: str = Option(None, "--vt-key", "-k", help="VirusTotal API Key - can be passed via VT_KEY env"),
        comment: str = Option("", "--comment", "-c", help="Comment for new MISP objects."),
        limit: int = Option(100, "--limit", "-l", help="Limit of VirusTotal objects to receive"),
        limit_relations: int = Option(
            40,
            "--limit-relations",
            "-L",
            help="Limit the amount of related objects. Note that this is for every relation queries."),
        relations: str = Option(
            "",
            "--relations",
            "-r",
            help=f"Relations to resolve via VirusTotal, available relations are: {', '.join(lib.all_relations)}"
        ),
        detections: int = Option(0, "--detections", "-d",
                                 help="Amount of detections a related VirusTotal object must at least have"),
        extract_domains: bool = Option(False, "--extract-domains", "-D",
                                       help="Extract domains from URL objects and add them as related object."),
        include_api_submissions: bool = Option(
            False, "-A", "--include-api", help="Include submission via api for submissions relation."
        ),
        filter: List[str] = Option(
            [],
            "--filter",
            "-f",
            help="Filtering related objects by matching this string(s) against json dumps of the objects."
        ),
        pivot: str = Option(
            None,
            "-p",
            "--pivot",
            help="Pivot from the given query before resolving relationships. This must be a valid VT file relation "
                 f"({', '.join(lib.file_relations)})."
        ),
        pivot_limit: int = Option(40, "-P", "--pivot-limit", help="Limit the amount of files returned by a pivot."),
        pivot_comment: str = Option(None, "-C", "--pivot-comment", help="Comment to add to the initial pivot object."),
        pivot_relationship: str = Option(
            "related-to",
            "--pivot-relationship",
            help="MISP relationship type for the relation between the initial pivot object and the results."
        )
):
    """
    Query VT for files and add them to a MISP event
    """
    if not url:
        url = os.getenv("MISP_URL", None)

    if not key:
        key = os.getenv("MISP_KEY", None)

    if not vt_key:
        vt_key = os.getenv("VT_KEY", None)

    if not url or not key or not vt_key:
        error("URL and key must be given either through param or env.")

    if pivot and pivot not in lib.file_relations:
        error("Pivot relationship is not valid or not implemented.")

    misp = PyMISP(url, key)
    misp.global_pythonify = True
    event = misp.get_event(uuid)
    results = lib.vt_query(
        api_key=vt_key,
        query=query,
        limit=limit
    )

    pivot_object = None
    if pivot:
        pivot_results = []
        for r in results:
            pivot_results.extend(
                lib.pivot_from_hash(
                    api_key=vt_key,
                    sha256_hash=r["attributes"]["sha256"],
                    rel=pivot,
                    limit=pivot_limit,
                    disable_output=state["quiet"]
                )
            )
        if len(pivot_results) == 0:
            error("[PIV] No files returned.")
            raise typer.Exit(-1)
        else:
            pivot_object = lib.process_results(
                results=results,
                event=event,
                comment=pivot_comment,
                disable_output=state["quiet"],
                extract_domains=False
            )[0]
            results = pivot_results

    created_objects = lib.process_results(
        results=results,
        event=event,
        comment=comment,
        disable_output=state["quiet"],
        extract_domains=extract_domains
    )
    lib.process_relations(
        api_key=vt_key,
        objects=created_objects,
        event=event,
        relations_string=relations,
        detections=detections,
        disable_output=state["quiet"],
        extract_domains=extract_domains,
        limit=limit_relations,
        filter=filter,
        include_api_submissions=include_api_submissions
    )
    if pivot and pivot_object:
        for obj in created_objects:
            lib.add_reference(pivot_object, obj.uuid, pivot_relationship)
    event.published = False
    misp.update_event(event)


@app.callback()
def callback(
        quiet: bool = Option(False, "-q", "--quiet", help="No output except stderr"),
        verbose: bool = Option(False, "-v", "--verbose", help="Use verbose logging")
):
    basicConfig(
        level="DEBUG" if verbose else "INFO", format="%(message)s", datefmt="[%X]", handlers=[RichHandler()]
    )
    state["quiet"] = quiet


app.add_typer(notifications.app, name="notifications")
app.add_typer(notifications.app, name="no")
app.add_typer(retrohunts.app, name="retrohunts")
app.add_typer(retrohunts.app, name="re")

if __name__ == "__main__":
    app()



import os
from typing import List

import typer
from pymisp import PyMISP
from rich.box import MINIMAL
from rich.console import Console
from rich.table import Table

from vt2m.lib.lib import warning, error, get_vt_notifications, process_results, process_relations
from vt2m.lib.output import add_object_to_table

app = typer.Typer(help="Query and process VT notifications")


@app.command("list")
def list_notifications(
        vt_key: str = typer.Option(None, "-k", "--vt-key", help="VT API Key - can also be set via VT_KEY env"),
        filter: str = typer.Option("", "-f", "--filter", help="Filter to be used for filtering notifications"),
        limit: int = typer.Option(10, "-l", "--limit", help="Amount of notifications to grab"),
        sha256: bool = typer.Option(False, "-s", "--sha256", help="Only show sha256 hashes")
):
    """List currently available VirusTotal notifications"""
    con = Console()
    if not vt_key:
        vt_key = os.getenv("VT_KEY")

    if not all([vt_key]):
        error("Not all required parameters were given.")
        raise typer.Exit(-1)

    notifications = get_vt_notifications(
        vt_key=vt_key,
        filter=filter,
        limit=limit
    )

    if len(notifications) == 0:
        warning("No notifications found.")
        raise typer.Exit(1)

    if sha256:
        for notification in notifications:
            con.print(notification["attributes"]["sha256"])
    else:
        t = Table(box=MINIMAL)
        t.add_column("Rule Name")
        t.add_column("First Seen")
        t.add_column("SHA256")
        for notification in notifications:
            add_object_to_table(
                t, notification,
                "context_attributes.rule_name", "attributes.first_submission_date", "attributes.sha256"
            )
        con.print(t)


@app.command("import")
def import_notifications(
        vt_key: str = typer.Option(None, help="VT API Key - can also be set via VT_KEY env"),
        filter: str = typer.Option("", help="Filter to be used for filtering notifications"),
        limit: int = typer.Option(10, help="Amount of notifications to grab"),
        uuid: str = typer.Option(..., "--uuid", "-u", help="MISP event UUID"),
        url: str = typer.Option(None, "--url", "-U", help="MISP URL - can be passed via MISP_URL env"),
        key: str = typer.Option(None, "--key", "-K", help="MISP API Key - can be passed via MISP_KEY env"),
        comment: str = typer.Option("", "--comment", "-c", help="Comment for new MISP objects"),
        relations: str = typer.Option("", "--relations", "-r", help="Relations to resolve via VirusTotal"),
        detections: int = typer.Option(0, "--detections", "-d",
                                       help="Amount of detections a related VirusTotal object must at least have"),
        extract_domains: bool = typer.Option(False, "--extract-domains", "-D",
                                             help="Extract domains from URL objects and add them as related object"),
        relation_filter: List[str] = typer.Option([], "--filter", "-f",
                                                  help="Filtering related objects by matching this string(s) "
                                                       "against json dumps of the objects"),
        quiet: bool = typer.Option(False, "--quiet", "-q", help="Disable output")
):
    """Import files related to notifications"""
    if not url:
        url = os.getenv("MISP_URL", None)

    if not key:
        key = os.getenv("MISP_KEY", None)

    if not vt_key:
        vt_key = os.getenv("VT_KEY", None)

    if not url or not key or not vt_key:
        error("URL and key must be given either through param or env.")
        raise typer.Exit(-1)

    misp = PyMISP(url, key)
    misp.global_pythonify = True
    event = misp.get_event(uuid)

    files = get_vt_notifications(
        vt_key=vt_key,
        filter=filter,
        limit=limit
    )
    created_objects = process_results(
        results=files,
        event=event,
        comment=comment,
        disable_output=quiet,
        extract_domains=extract_domains
    )
    process_relations(
        api_key=vt_key,
        objects=created_objects,
        event=event,
        relations_string=relations,
        detections=detections,
        disable_output=quiet,
        extract_domains=extract_domains,
        filter=relation_filter
    )
    event.published = False
    misp.update_event(event)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['vt2m', 'vt2m.lib', 'vt2m.subcommands']

package_data = \
{'': ['*']}

install_requires = \
['pymisp>=2.4.169.3,<3.0.0.0',
 'requests>=2.27.1,<3.0.0',
 'rich>=13.3.5,<14.0.0',
 'typer>=0.9.0,<0.10.0',
 'vt-py>=0.17.4,<0.18.0']

entry_points = \
{'console_scripts': ['vt2m = vt2m.main:app']}

setup_kwargs = {
    'name': 'vt2m',
    'version': '0.1.14',
    'description': 'Automatically import results from VirusTotal queries into MISP objects',
    'long_description': '# VirusTotal Query to MISP Objects (vt2m)\n\nWhile there are multiple Python projects which implement the object creation based on single VirusTotal objects, this\nproject aims to enable users to directly convert VirusTotal search queries to MISP objects.\n**This is work in progress.** Future release will implement handling URLs, Domain and IP objects, too. Right now, only\nfile objects - as a base for queries - are implemented. These file objects can have related IPs, domains and URLs,\nthough.\n\n## Installation\n\n```\npip install vt2m\n```\n\n## Usage\n\nIf you use the script frequently, passing the arguments as environment variables (`MISP_URL`, `MISP_KEY`, `VT_KEY`)\ncan be useful to save some time. For example, this can be achieved through creating a shell script which passes the\nenvironment variables and executes the command with spaces in front, so it does not show up in the shell history.\nSomething like this:\n\n```bash\n#!/usr/bin/env bash\n\nSAVEIFS=$IFS\nIFS=$(echo -en "\\n\\b")\nIFS=$SAVEIFS\n    MISP_URL="https://my.misp.host.local" MISP_KEY="MyMISPApiKey1234567890" VT_KEY="MyVTApiKey1234567890" /path/to/venv/bin/vt2m "$@"\nIFS=$SAVEIFS\n```\n\nChanging the IFS is a must, so spaces are not seen as a field seperator.\n\nOverall, `vt2m` supports three commands:\n\n- VirusTotal Intelligence Search via `query`\n- Accessing Live Hunting notifications via `notifications` (or `no`)\n- Accessing Retrohunt results via `retrohunts` (or `re`)\n\n### VirusTotal Ingelligence Search: `query`\n\n```\nUsage: vt2m query [OPTIONS] QUERY\n\n  Query VT for files and add them to a MISP event\n\nArguments:\n  QUERY  VirusTotal Query  [required]\n\nOptions:\n  -u, --uuid TEXT                MISP event UUID  [required]\n  -U, --url TEXT                 MISP URL - can be passed via MISP_URL env\n  -K, --key TEXT                 MISP API Key - can be passed via MISP_KEY env\n  -k, --vt-key TEXT              VirusTotal API Key - can be passed via VT_KEY\n                                 env\n  -c, --comment TEXT             Comment for new MISP objects.\n  -l, --limit INTEGER            Limit of VirusTotal objects to receive\n                                 [default: 100]\n  -L, --limit-relations INTEGER  Limit the amount of related objects. Note\n                                 that this is for every relation queries.\n                                 [default: 40]\n  -r, --relations TEXT           Relations to resolve via VirusTotal,\n                                 available relations are: execution_parents,\n                                 compressed_parents, bundled_files,\n                                 dropped_files, contacted_urls, embedded_urls,\n                                 itw_urls, contacted_domains,\n                                 embedded_domains, itw_domains, contacted_ips,\n                                 embedded_ips, itw_ips, submissions,\n                                 communicating_files\n  -d, --detections INTEGER       Amount of detections a related VirusTotal\n                                 object must at least have  [default: 0]\n  -D, --extract-domains          Extract domains from URL objects and add them\n                                 as related object.\n  -f, --filter TEXT              Filtering related objects by matching this\n                                 string(s) against json dumps of the objects.\n  -p, --pivot TEXT               Pivot from the given query before resolving\n                                 relationships. This must be a valid VT file\n                                 relation (execution_parents,\n                                 compressed_parents, bundled_files,\n                                 dropped_files).\n  -P, --pivot-limit INTEGER      Limit the amount of files returned by a\n                                 pivot.  [default: 40]\n  -C, --pivot-comment TEXT       Comment to add to the initial pivot object.\n  --pivot-relationship TEXT      MISP relationship type for the relation\n                                 between the initial pivot object and the\n                                 results.  [default: related-to]\n  --help                         Show this message and exit.\n```\n\nThe `query` command supports ingesting files from a VT search, but additional also requesting specific related files or\ninfrastructure indicators (via `--relations`) and an initial pivot off the files (via `--pivot`). The latter means that,\ne.g., you\'re able to search for files that are commonly dropped or contained within the samples you\'re actually\nsearching for and use the "parent" files as your regular result set, enrichting them with additional relationships etc.\n\nVia `--relations` VirusTotal relations can be resolved and added as MISP objects with the specific relations, e.g. the\nfollowing graph was created using vt2m:\n![MISP Graph](.github/screenshots/graph.png)\n*Graph created\nvia `vt2m --uuid <UUID> --limit 5 --relations dropped_files,execution_parents "behaviour_processes:\\"ping -n 70\\""`*\n\n### VirusTotal Livehunt notifications: `notifications`\n\n```\nUsage: vt2m notifications [OPTIONS] COMMAND [ARGS]...\n\n  Query and process VT notifications\n\nOptions:\n  --help  Show this message and exit.\n\nCommands:\n  import  Import files related to notifications\n  list    List currently available VirusTotal notifications\n```\n\nThe command allows to list and to import livehunt results via two subcommands.\n\n### VirusTotal Retrohunt results: `retrohunts`\n\n```\nUsage: vt2m retrohunts [OPTIONS] COMMAND [ARGS]...\n\n  Query for retrohunt results.\n\nOptions:\n  --help  Show this message and exit.\n\nCommands:\n  import  Imports results of a retrohunt into a MISP event\n  list    Lists available retrohunts\n```\n\nThe command allows to list and to import retrohunt results via two subcommands.\n\n## Examples\n\n### Query for hashes\n\nIn order to just ingest files you already found via VirusTotal search, you can query for the file hashes in order to\nsave VirusTotal queries. This way the command only counts towards regular API calls.\n\n`vt2m query --uuid <MISP Event UUID> "<hash 1> <hash 2> <hash 3> ... <hash n>"`\n\nOf course, the same way you\'re able to include related objects, e.g. contacted URLs, the according domains and dropped\nfiles during execution.\n\n`vt2m query --uuid <MISP Event UUID> --relations contacted_urls,dropped_files --extract-domains "<hash 1> <hash 2> <hash 3> ... <hash n>"`\n\n### Query using VirusTotal Intelligence Searches\n\nSimilar as above, you can directly use VirusTotal search queries for ingesting indicators into MISP events:\n\n`vt2m query --uuid <MISP Event UUID> --relations contacted_urls,dropped_files --extract-domains "imphash:<imphash>"`\n\n### Pivot before query\n\nSometimes it\'s necessary to pivot before receiving the actual files. This is useful, if files drop a common file during\nexecution, or archives have a common file bundled. This can be done this way:\n\n`vt2m query --uuid <MISP Event UUID> --relations contacted_domains,bundled_files --detections 3 --pivot compressed_parents --pivot-comment "RC4 Key" --pivot-limit 20 --pivot-relationship contained-within 99c9440a84cdc428ce140de901452eb334faec49f1f6258acdde1ddcbb34376e`\n\nAs this command introduces some parameters that might be not self-explanatory, here is a small breakdown of them:\n\n| Parameter                              | Description                                                                         |\n|----------------------------------------|-------------------------------------------------------------------------------------|\n| `--uuid`                               | The UUID of the MISP event which will be the ingestion target                       |\n| `--relations`                          | VirusTotal relations to query for                                                   |\n| `--detections 3`                       | Filter for related objects with at least 3 AV detections                            |\n| `--pivot compressed_parents`           | Enable pivot mode and pivot via the relation given                                  |\n| `--pivot-comment`                      | Comment to add to the MISP object pivoted from                                      |\n| `--pivot-limit 20`                     | Limit the pivot to 20 objects                                                       |\n| `--pivot-relationship contaned-within` | MISP relationship between the pivoted object and the results, default is related-to |\n\nThe above query is resulting in the following MISP graph:\n\n![MISP graph of the pivot example](.github/screenshots/graph2.png)',
    'author': '3c7',
    'author_email': '3c7@posteo.de',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/3c7/vt2m',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



from datetime import datetime
from logging import getLogger
from typing import Dict

from rich.table import Table

LOGGER = getLogger("vt2m")


def debug(*args, **kwargs) -> None:
    """Wrapper for Logger.debug."""
    LOGGER.debug(*args, **kwargs)


def info(*args, **kwargs) -> None:
    """Wrapper for Logger.info."""
    LOGGER.info(*args, **kwargs)


def print(*args, **kwargs) -> None:
    """Shadows python print method in order to use typer.echo instead."""
    LOGGER.info(*args, **kwargs)


def warning(*args, **kwargs) -> None:
    """Wrapper for Logger.warning"""
    LOGGER.warning(*args, **kwargs)


def error(*args, **kwargs):
    """Wrapper for Logger.error"""
    LOGGER.error(*args, **kwargs)


def add_object_to_table(t: Table, obj: Dict, *attributes) -> None:
    row = []
    for attrib in attributes:
        keys = attrib.split(".")
        tmp = obj
        for idx, key in enumerate(keys):
            try:
                if idx == len(keys) - 1 and "date" in key:
                    try:
                        tmp = datetime.fromtimestamp(tmp[key]).isoformat()
                    except:
                        tmp = tmp[key]
                        warning(f"Could not parse {tmp} ({key}) as date.")
                else:
                    tmp = tmp[key]
            except KeyError as ke:
                tmp = f"<{ke} not found>"
        if isinstance(tmp, list):
            tmp = ", ".join(tmp)
        row.append(str(tmp))
    t.add_row(*row)


def print_file_object(obj: Dict, *attributes: str) -> None:
    """Print file object with given attributes. Attributes are given in a list of strings which can end with a reserved
    length passed to the format string, e.g., `attributes.sha256,40`."""
    for idx, attrib in enumerate(attributes):
        tmp = obj
        keys = attrib.split(".")
        if "," in keys[-1]:
            keys[-1], length = keys[-1].split(",", maxsplit=1)
        else:
            length = None

        try:
            for idx2, key in enumerate(keys):
                if idx2 == len(keys) - 1 and "date" in key:
                    try:
                        tmp = datetime.fromtimestamp(tmp[key]).isoformat()
                    except:
                        warning(f"[WARN] Tried to parse {keys} as date, but was not successful.")
                        tmp = [key]
                else:
                    tmp = tmp[key]
        except KeyError:
            tmp = "<Not found>"
        if idx + 1 < len(attributes) and len(attributes) > 1:
            if length:
                print(f"{tmp:<{length}}", nl=False)
            else:
                print(tmp + " ", nl=False)
        else:
            if length:
                print(f"{tmp:<{length}}")
            else:
                print(tmp)
